package com.yunyan.web.controller.system;

import java.util.Arrays;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.yunyan.system.domain.dto.DeptManageAddDto;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.yunyan.common.annotation.Log;
import com.yunyan.common.core.controller.BaseController;
import com.yunyan.common.core.domain.AjaxResult;
import com.yunyan.common.enums.BusinessType;
import com.yunyan.system.domain.DeptManage;
import com.yunyan.system.service.IDeptManageService;
import com.yunyan.common.utils.poi.ExcelUtil;
import com.yunyan.common.core.page.TableDataInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.yunyan.common.core.domain.AjaxResultVo;
import com.yunyan.common.utils.SecurityUtils;
import java.util.Date;

/**
 * 管理部门与被管理部门的Controller
 *
 * @author yunyan
 * @date 2024-10-12
 */
@Api(tags = "管理部门与被管理部门的")
@RestController
@RequestMapping("/system/deptManage")
public class DeptManageController extends BaseController
{
    @Autowired
    private IDeptManageService deptManageService;

    @ApiOperation(value="批量新增管理部门与被管理部门的", notes="新增管理部门与被管理部门的")
    //@PreAuthorize("@ss.hasPermi('system:deptManage:add')")
    @Log(title = "管理部门与被管理部门的", businessType = BusinessType.INSERT)
    @PostMapping("/addList")
    public AjaxResultVo addList(@Validated @RequestBody DeptManageAddDto dto)
    {
        return deptManageService.addList(dto);
    }

    /**
     * 查询管理部门与被管理部门的列表
     */
    @ApiOperation(value="查询管理部门与被管理部门的列表", notes="查询管理部门与被管理部门的列表")
    @PreAuthorize("@ss.hasPermi('system:deptManage:list')")
    @GetMapping("/list")
    public TableDataInfo<DeptManage> list(DeptManage deptManage)
    {
        startPage();
        List<DeptManage> list = deptManageService.selectDeptManageList(deptManage);
        return getDataTable(list);
    }

    /**
     * 导出管理部门与被管理部门的列表
     */
    @PreAuthorize("@ss.hasPermi('system:deptManage:export')")
    @Log(title = "管理部门与被管理部门的", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DeptManage deptManage)
    {
        List<DeptManage> list = deptManageService.selectDeptManageList(deptManage);
        ExcelUtil<DeptManage> util = new ExcelUtil<DeptManage>(DeptManage.class);
        util.exportExcel(response, list, "管理部门与被管理部门的数据");
    }

    /**
     * 获取管理部门与被管理部门的详细信息
     */
    @ApiOperation(value="获取管理部门与被管理部门的详细信息", notes="获取管理部门与被管理部门的详细信息")
    @PreAuthorize("@ss.hasPermi('system:deptManage:query')")
    @GetMapping(value = "/detail/{deptIdManage}")
    public AjaxResultVo getInfo(@PathVariable("deptIdManage") Long deptIdManage)
    {
        return AjaxResultVo.success(deptManageService.getById(deptIdManage));
    }

    /**
     * 新增管理部门与被管理部门的
     */
    @ApiOperation(value="新增管理部门与被管理部门的", notes="新增管理部门与被管理部门的")
    @PreAuthorize("@ss.hasPermi('system:deptManage:add')")
    @Log(title = "管理部门与被管理部门的", businessType = BusinessType.INSERT)
    @PostMapping("/add")
    public AjaxResult add(@RequestBody DeptManage dto)
    {
        return toAjax(deptManageService.save(dto));
    }

    /**
     * 修改管理部门与被管理部门的
     */
    @ApiOperation(value="修改管理部门与被管理部门的", notes="修改管理部门与被管理部门的")
    @PreAuthorize("@ss.hasPermi('system:deptManage:edit')")
    @Log(title = "管理部门与被管理部门的", businessType = BusinessType.UPDATE)
    @PutMapping("/edit")
    public AjaxResult edit(@RequestBody DeptManage dto)
    {
        return toAjax(deptManageService.updateById(dto));
    }

    /**
     * 删除管理部门与被管理部门的
     */
    @ApiOperation(value="删除管理部门与被管理部门的", notes="删除管理部门与被管理部门的")
    @PreAuthorize("@ss.hasPermi('system:deptManage:remove')")
    @Log(title = "管理部门与被管理部门的", businessType = BusinessType.DELETE)
	@DeleteMapping("/remove/{deptIdManages}")
    public AjaxResult remove(@PathVariable Long[] deptIdManages)
    {
        return toAjax(deptManageService.removeByIds(Arrays.asList(deptIdManages)));
    }

}
