package com.yunyan.web.controller.system;

import java.util.List;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import com.yunyan.common.core.domain.model.UserAreaVo;
import com.yunyan.common.core.domain.AjaxResultVo;
import com.yunyan.common.core.domain.TreeSelect;
import com.yunyan.system.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import com.yunyan.common.annotation.Log;
import com.yunyan.common.core.controller.BaseController;
import com.yunyan.common.core.domain.AjaxResult;
import com.yunyan.common.core.domain.entity.SysDept;
import com.yunyan.common.core.domain.entity.SysRole;
import com.yunyan.common.core.domain.entity.SysUser;
import com.yunyan.common.core.page.TableDataInfo;
import com.yunyan.common.enums.BusinessType;
import com.yunyan.common.utils.SecurityUtils;
import com.yunyan.common.utils.StringUtils;
import com.yunyan.common.utils.poi.ExcelUtil;

/**
 * 用户信息
 *
 * @author yunyan
 */
@Api(tags = "6.0.1用户")
@RestController
@RequestMapping("/system/user")
public class SysUserController extends BaseController
{
    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private ISysPostService postService;

    @Autowired
    private IUserDeptService userDeptService;

    /**
     * 获取用户列表
     */
    @PreAuthorize("@ss.hasPermi('system:user:list')")
    @GetMapping("/list")
    public TableDataInfo list(SysUser user)
    {
        startPage();
        List<SysUser> list = userService.selectUserList(user);
        return getDataTable(list);
    }

    @ApiOperation(value="业务员选取列表(无权限限制)", notes="业务员选取列表(无权限限制)")
//    @PreAuthorize("@ss.hasPermi('system:user:list')")
    @GetMapping("/list/select")
    public TableDataInfo<SysUser> listForselect(SysUser user)
    {
        startPage();
        List<SysUser> list = userService.selectUserList(user);
        return getDataTable(list);
    }

    @Log(title = "用户管理", businessType = BusinessType.EXPORT)
    @PreAuthorize("@ss.hasPermi('system:user:export')")
    @PostMapping("/export")
    public void export(HttpServletResponse response, SysUser user)
    {
        List<SysUser> list = userService.selectUserList(user);
         ExcelUtil<SysUser> util = new ExcelUtil<SysUser>(SysUser.class);
        util.exportExcel(response, list, "用户数据");
    }

    @Log(title = "用户管理", businessType = BusinessType.IMPORT)
    @PreAuthorize("@ss.hasPermi('system:user:import')")
    @PostMapping("/importData")
    public AjaxResult importData(MultipartFile file, boolean updateSupport) throws Exception
    {
        ExcelUtil<SysUser> util = new ExcelUtil<SysUser>(SysUser.class);
        List<SysUser> userList = util.importExcel(file.getInputStream());
        String operName = getUsername();
        String message = userService.importUser(userList, updateSupport, operName);
        return success(message);
    }

    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response)
    {
        ExcelUtil<SysUser> util = new ExcelUtil<SysUser>(SysUser.class);
        util.importTemplateExcel(response, "用户数据");
    }

    /**
     * 根据用户编号获取详细信息
     */
    @ApiOperation(value="根据用户编号获取详细信息", notes="根据用户编号获取详细信息")
    @PreAuthorize("@ss.hasPermi('system:user:query')")
    @GetMapping(value = { "/", "/{userId}" })
    public AjaxResult getInfo(@PathVariable(value = "userId", required = false) Long userId)
    {
        userService.checkUserDataScope(userId);
        AjaxResult ajax = AjaxResult.success();
        List<SysRole> roles = roleService.selectRoleAll();
        ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter(r -> !r.isAdmin()).collect(Collectors.toList()));
        ajax.put("posts", postService.selectPostAll());
        if (StringUtils.isNotNull(userId))
        {
            SysUser sysUser = userService.selectUserById(userId);
            //设置用户的部门
            List<Long> deptIdList = userDeptService.getDeptIdList(userId);
            sysUser.setDeptIds(deptIdList.toArray(new Long[deptIdList.size()]));
            ajax.put(AjaxResult.DATA_TAG, sysUser);
            ajax.put("postIds", postService.selectPostListByUserId(userId));
            ajax.put("roleIds", sysUser.getRoles().stream().map(SysRole::getRoleId).collect(Collectors.toList()));

            //ajax.put("deptIds", userDeptService.getDeptIdList(userId));
        }
        return ajax;
    }

    /**
     * 新增用户
     */
    @ApiOperation(value="新增用户/业务员", notes="新增用户/业务员")
    @PreAuthorize("@ss.hasPermi('system:user:add')")
    @Log(title = "用户管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysUser user)
    {
        if (!userService.checkUserNameUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !userService.checkPhoneUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，手机号码已存在");
        }
        else if (StringUtils.isNotEmpty(user.getEmail()) && !userService.checkEmailUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        user.setCreateBy(getUsername());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        return toAjax(userService.insertUser(user));
    }

    /**
     * 修改用户
     */
    @ApiOperation(value="修改用户/业务员", notes="修改用户/业务员")
    @PreAuthorize("@ss.hasPermi('system:user:edit')")
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysUser user)
    {
        userService.checkUserAllowed(user);
        userService.checkUserDataScope(user.getUserId());
        if (!userService.checkUserNameUnique(user))
        {
            return error("修改用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !userService.checkPhoneUnique(user))
        {
            return error("修改用户'" + user.getUserName() + "'失败，手机号码已存在");
        }
        else if (StringUtils.isNotEmpty(user.getEmail()) && !userService.checkEmailUnique(user))
        {
            return error("修改用户'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        user.setUpdateBy(getUsername());
        return toAjax(userService.updateUser(user));
    }

    /**
     * 删除用户
     */
    @PreAuthorize("@ss.hasPermi('system:user:remove')")
    @Log(title = "用户管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{userIds}")
    public AjaxResult remove(@PathVariable Long[] userIds)
    {
        if (ArrayUtils.contains(userIds, getUserId()))
        {
            return error("当前用户不能删除");
        }
        return toAjax(userService.deleteUserByIds(userIds));
    }

    /**
     * 重置密码
     */
    @PreAuthorize("@ss.hasPermi('system:user:resetPwd')")
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping("/resetPwd")
    public AjaxResult resetPwd(@RequestBody SysUser user)
    {
        userService.checkUserAllowed(user);
        userService.checkUserDataScope(user.getUserId());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        user.setUpdateBy(getUsername());
        return toAjax(userService.resetPwd(user));
    }

    /**
     * 状态修改
     */
    @PreAuthorize("@ss.hasPermi('system:user:edit')")
    @Log(title = "用户管理", businessType = BusinessType.UPDATE)
    @PutMapping("/changeStatus")
    public AjaxResult changeStatus(@RequestBody SysUser user)
    {
        userService.checkUserAllowed(user);
        userService.checkUserDataScope(user.getUserId());
        user.setUpdateBy(getUsername());
        return toAjax(userService.updateUserStatus(user));
    }

    /**
     * 根据用户编号获取授权角色
     */
    @PreAuthorize("@ss.hasPermi('system:user:query')")
    @GetMapping("/authRole/{userId}")
    public AjaxResult authRole(@PathVariable("userId") Long userId)
    {
        AjaxResult ajax = AjaxResult.success();
        SysUser user = userService.selectUserById(userId);
        List<SysRole> roles = roleService.selectRolesByUserId(userId);
        ajax.put("user", user);
        ajax.put("roles", SysUser.isAdmin(userId) ? roles : roles.stream().filter(r -> !r.isAdmin()).collect(Collectors.toList()));
        return ajax;
    }

    /**
     * 用户授权角色
     */
    @PreAuthorize("@ss.hasPermi('system:user:edit')")
    @Log(title = "用户管理", businessType = BusinessType.GRANT)
    @PutMapping("/authRole")
    public AjaxResult insertAuthRole(Long userId, Long[] roleIds)
    {
        userService.checkUserDataScope(userId);
        userService.insertUserAuth(userId, roleIds);
        return success();
    }

    /**
     * 获取部门树列表
     */
    @PreAuthorize("@ss.hasPermi('system:user:list')")
    @GetMapping("/deptTree")
    public AjaxResult deptTree(SysDept dept)
    {
        return success(deptService.selectDeptTreeList(dept));
    }


    /**
     * 获取部门树列表
     */
    @ApiOperation(value="获取部门树列表(选取列表(无权限限制))", notes="部门树列表(选取列表(无权限限制))")
    @GetMapping("/deptTree/select")
    public AjaxResultVo<List<TreeSelect>> deptTreeSelect(SysDept dept)
    {
        return AjaxResultVo.success(deptService.selectDeptTreeList(dept));
    }

    /*@ApiOperation(value="填报日期, 获取部门树列表(选取列表(无权限限制))", notes="部门树列表(选取列表(无权限限制))")
    @GetMapping("/deptTree/fillDate/select")
    public AjaxResultVo<List<TreeSelect>> deptTreeSelectFillDate(SysDept dept)
    {
        if(null != dept.getSelectFillDateType() && 1 == dept.getSelectFillDateType() ) {
            List<Long> deptIdList = SecurityUtils.getDeptIdList();
            if(!deptIdList.isEmpty()) {
                SysDept sysDept = deptService.getById(deptIdList.get(0));
                if(null != sysDept) {
                    if(sysDept.getDeptLevel() == 2) {
                        dept.getParams().put("dept1", "dept1");
                        dept.getParams().put("deptIdCurrent", sysDept.getDeptId());
                        dept.getParams().put("deptLevelSelect", 8);
                    } else if(sysDept.getDeptLevel() == 2) {
                        dept.getParams().put("deptLevel8", 1);
                        dept.getParams().put("deptParentIdCurrent", sysDept.getParentId());
                    }
                }
            }
        }

        return AjaxResultVo.success(deptService.selectDeptTreeList(dept));
    }*/

    @ApiOperation(value="填报日期, 获取部门列表", notes="部门列表")
    @GetMapping("/deptTree/fillDate/list")
    public TableDataInfo<List<SysDept>> deptTreeSelectFillDateList(SysDept dept)
    {

        /*List<Long> deptIdList = SecurityUtils.getDeptIdList();
        if(!deptIdList.isEmpty()) {
            SysDept sysDept = deptService.getById(deptIdList.get(0));
            if(null != sysDept) {
                if(sysDept.getDeptLevel() == 2) {
                    dept.getParams().put("level1", sysDept.getDeptId());
                    dept.getParams().put("deptParentIdCurrent", sysDept.getDeptId());
                    //dept.getParams().put("deptIdCurrent", sysDept.getDeptId());
                    dept.getParams().put("deptLevelSelect", 8);
                } else if(sysDept.getDeptLevel() == 8) {
                    dept.getParams().put("level2", sysDept.getDeptId());
                    dept.getParams().put("deptParentIdCurrent", sysDept.getParentId());
                    dept.getParams().put("deptIdCurrent", sysDept.getDeptId());
                }
            }
            if(null == dept.getParams().get("level1") && null == dept.getParams().get("level2") ) {
                dept.setDeptId(-1L);
            }
        } else {
            dept.setDeptId(-1L);
        }*/

        startPage();
        List<SysDept> list = deptService.selectDeptListByFillDate(dept);
        return getDataTable(list);

        //return AjaxResultVo.success();
    }

}
