package com.yunyan.business.document.y23.finalData;

import cn.hutool.core.util.NumberUtil;
import com.yunyan.business.constant.TemplateEpType;
import com.yunyan.business.document.utils.ReadExcelUtil;
import com.yunyan.business.document.y22.FinalY22Utils;
import com.yunyan.business.document.y22.pojo.finalExcelY22.FinalP3T1TotalItemVo;
import com.yunyan.business.document.y23.budget.pojo.budgetY23.BudgetP3ChangeItemVo;
import com.yunyan.business.document.y23.finalData.pojo.ChangeValueItemVo;
import com.yunyan.common.utils.NumberFormatUtils;

import java.math.BigDecimal;

/**
 * 决算的凭祥模板的读取(平南的 也有)
 */
public class FinalY23PxUtils {

    /**
     * 检查 是否是模板的 修改
     * @param templateExportType
     * @return 当不属于时, 返回true
     */
    private static boolean checkTmplateType(String templateExportType) {
        if(!TemplateEpType.Y2023FT1PX.getCode().equals(templateExportType) ) {
            return true;
        }

        return false;
    }

    /**
     * 第3部分的第一的收入的设置
     * @param incomeItem
     * @param templateExportType
     */
    public static void setP3T1ItemInfo(FinalP3T1TotalItemVo incomeItem, String templateExportType) {
        if(checkTmplateType(templateExportType)) {
            return;
        }

        String info = incomeItem.getInfo();
        if(incomeItem.getName().startsWith("一般公共预算财政") ||  incomeItem.getName().startsWith("政府性基金预算") || incomeItem.getName().startsWith("国有资本经营预算")) {
            info += "，为凭祥市本级财政当年拨付的资金";
            incomeItem.setInfo(info);
        }
        /*if (incomeItem.getName().startsWith("国有资本经营预算")) {
            info += "，为市本级财政当年拨付的资金";
            incomeItem.setInfo(info);
        }*/
        if (incomeItem.getName().startsWith("年初结转和结余")) {
            incomeItem.setName(incomeItem.getName().replaceFirst("年初结转和结余", "上年结转和结余"));
            info += "，既包括财政拨款结转和结余，也包括事业收入、经营收入、其他收入的结转和结余";
            //改成 年初
            info = info.replaceFirst("年初结转和结余", "上年结转和结余");
            incomeItem.setInfo(info);
        }

        if (incomeItem.getName().startsWith("其他收入")) {
            info += "，为预算单位在“财政拨款收入”“事业收入”“经营收入”之外取得的收入";
            incomeItem.setInfo(info);
            incomeItem.setInfoDescFlag(0);
        }

        if (incomeItem.getName().startsWith("事业收入")) {
            info += "，为事业单位开展业务活动取得的收入";
            incomeItem.setInfo(info);
            incomeItem.setInfoDescFlag(0);
        }
    }
    /**
     * 第3部分的第一的支出的设置
     * @param itemInfo
     * @param templateExportType
     */
    public static void setP3T1ItemInfo2(FinalP3T1TotalItemVo itemInfo, String templateExportType) {
        if(checkTmplateType(templateExportType)) {
            return;
        }

        String info = itemInfo.getInfo();
        if (itemInfo.getName().startsWith("结余分配")) {
            info += "，为事业单位按规定提取的职工福利基金、事业基金和缴纳的所得税等";
            itemInfo.setInfo(info);
        }
        if (itemInfo.getName().startsWith("年末结转和结余")) {
            info += "，为本年度或以前年度预算安排、因客观条件发生变化无法按原计划实施，需要延迟到以后年度按有关规定继续使用的资金，既包括财政拨款结转和结余，也包括事业收入、经营收入、其他收入的结转和结余";
            itemInfo.setInfo(info);
            itemInfo.setInfoDescFlag(0);
        }

        if (itemInfo.getName().startsWith("事业收入")) {
            //单位自行填写
            itemInfo.setInfoDescFlag(1);
        }
    }


    /**
     * 附属单位上缴 去除, 去除 返回 false
     * @param incomeItem
     * @param templateExportType
     * @return
     */
    public static boolean checkAddP3T1Item(FinalP3T1TotalItemVo incomeItem, String templateExportType) {
        if(!checkTmplateType(templateExportType) && incomeItem.getName().startsWith("附属单位上缴")) {
            return false;
        }

        return true;
    }

    /**
     * 增加（减少）0万元，增长（下降）0%
     * 比较 val1 与 val2, val1 >= val2 时, 返回:  增加（减少）  万元，增长（下降）  %
     * @param val1
     * @param val2
     * @return
     */
    public static String getAddOrSubStringP3t6CompareStr(String val1, String val2) {

        String val1NumNoformat = FinalY22Utils.strFormaTtoNum(val1);
        String val2NumNoformat = FinalY22Utils.strFormaTtoNum(val2);

        //数值变化
        BigDecimal subRes = NumberUtil.sub(val1NumNoformat, val2NumNoformat);

        String changeValue = subRes.abs().toString();
        String changeRatio = ReadExcelUtil.getChangeRatioValue(val2NumNoformat, subRes);
        String changeValueWord = ReadExcelUtil.getChangeValueWord(subRes);
        String changeRatioWord = ReadExcelUtil.getChangeRatioWord(subRes);

        String str = String.format("%s%s万元", changeValueWord, NumberFormatUtils.addThousandSeparatorForInteger(changeValue));

        return str;
    }

    /**
     * 增长（下降）0%
     * 比较 val1 与 val2, val1 >= val2 时, 返回:  增加（减少）  万元，增长（下降）  %
     * @param val1
     * @param val2
     * @return
     */
    public static String getAddOrSubStringP3t6CompareStrRatio(String val1, String val2) {

        String val1NumNoformat = FinalY22Utils.strFormaTtoNum(val1);
        String val2NumNoformat = FinalY22Utils.strFormaTtoNum(val2);

        //数值变化
        BigDecimal subRes = NumberUtil.sub(val1NumNoformat, val2NumNoformat);

        String changeValue = subRes.abs().toString();
        String changeRatio = ReadExcelUtil.getChangeRatioValue(val2NumNoformat, subRes);
        String changeValueWord = ReadExcelUtil.getChangeValueWord(subRes);
        String changeRatioWord = ReadExcelUtil.getChangeRatioWord(subRes);

        String str = String.format("%s%s%%", changeRatioWord, changeRatio);

        return str;
    }




}
