package com.yunyan.framework.web.service;

import com.yunyan.common.core.domain.entity.SysDept;
import com.yunyan.system.service.IUserDeptService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import com.yunyan.common.core.domain.entity.SysUser;
import com.yunyan.common.core.domain.model.LoginUser;
import com.yunyan.common.enums.UserStatus;
import com.yunyan.common.exception.ServiceException;
import com.yunyan.common.utils.StringUtils;
import com.yunyan.system.service.ISysUserService;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户验证处理
 *
 * @author yunyan
 */
@Service
public class UserDetailsServiceImpl implements UserDetailsService
{
    private static final Logger log = LoggerFactory.getLogger(UserDetailsServiceImpl.class);

    @Autowired
    private ISysUserService userService;
    
    @Autowired
    private SysPasswordService passwordService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private IUserDeptService userDeptService;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException
    {
        SysUser user = userService.selectUserByUserName(username);
        if (StringUtils.isNull(user))
        {
            log.info("登录用户：{} 不存在.", username);
            throw new ServiceException("登录用户：" + username + " 不存在");
        }
        else if (UserStatus.DELETED.getCode().equals(user.getDelFlag()))
        {
            log.info("登录用户：{} 已被删除.", username);
            throw new ServiceException("对不起，您的账号：" + username + " 已被删除");
        }
        else if (UserStatus.DISABLE.getCode().equals(user.getStatus()))
        {
            log.info("登录用户：{} 已被停用.", username);
            throw new ServiceException("对不起，您的账号：" + username + " 已停用");
        }

        passwordService.validate(user);

        return createLoginUser(user);
    }

    public UserDetails createLoginUser(SysUser user)
    {
        List<Long> deptIdList = userDeptService.getDeptIdList(user.getUserId());
        /*List<Long> deptIdList = new ArrayList<>();
        if( null != user.getDeptList() && !user.getDeptList().isEmpty()) {
            deptIdList = user.getDeptList().stream().map(SysDept::getDeptId).collect(Collectors.toList());
        }*/
        return new LoginUser(user.getUserId(), deptIdList, user, permissionService.getMenuPermission(user));
    }
}
